############################################################################
# add field index shelves to flat-file database mechanism
# to optimize "index only" displays, use classes at end of this file;
# change browe, index, submit to use new loaders for "Index only" mode;
# minor nit: uses single lock file for all index shelve read/write ops;
# storing record copies instead of filenames in index shelves would be
# slightly faster (avoids opening flat files), but would take more space;
############################################################################

import sys; sys.path.append('..')      # see parent dir from Admin
from Mutex import mutexcntl
import dbfiles, shelve, pickle, string, sys

class Dbase(mutexcntl.MutexCntl, dbfiles.Dbase):
    def indexName(self, field):
        return self.dirname + strig.replace(fieldname, ' ', '-')

    def makeKey(self):
        if not hasattr(self, 'cacheKey'):                 # cache filename
            self.cacheKey = dbfiles.Dbase.makeKey(self)   # need it here too
        return self.cacheKey

    def safeWriteIndex(self, fieldname, newdata, recfilename):
        index = shelve.open(indexName(fieldname))
        try:
            keyval  = newdata[fieldname]                  # recs have all fields
            reclist = index[keyval]                       # fetch, mod, rewrite
            reclist.append(recfilename)                   # add to current list
            index[keyval] = reclist
        except KeyError:
            index[keyval] = [recfilename]                 # add to new list

    def safeLoadKeys(self, fieldname):
        if fieldname in self.indexfields:
            keys = shelve.open(indexName(fieldname)).keys()
            keys.sort()
        except:
            keys, index = dbfiles.loadIndexedTable(self, fieldname)
        return keys

    def safeLoadByKey(self, fieldname, fieldvalue):
        if fieldname in self.indexfields:
            index   = shelve.open(indexName(fieldname))
            try:
                index = dbase[fieldvalue]
                reports = []
                for filename in index:
                    reports.append(pickle.load(open(filename, 'r')))
                return reports    
            except KeyError:
                return []
        else:
            key, index = dbfiles.loadIndexedTable(self, fieldname)
            try:
                return index[fieldvalue]
            except KeyError:
                return []

    # top-level interfaces (plus dbcommon and dbfiles)

    def writeItem(self, newdata):                 
        # extend to update indexes
        filename = self.makeKey()
        dbfiles.Dbase.writeItem(self, newdata)
        for fieldname in self.indexfields:
            self.exclusiveAction(self.safeWriteIndex, fieldname, newdata)             

    def loadKeysList(self, fieldname):            
        # load field's keys list only
        self.sharedAction(self.safeLoadKeysList, fieldname)

    def loadByKey(self, fieldname, fieldvalue):   
        # load matching recs lisy only
        return self.safeAction(self.safeLoadByKey, fieldname, fieldvalue)

class DbaseErrata(Dbase):
    dirname     = 'DbaseFilesIndexed/errataDB/'
    filename    = dirname + 'indexes'
    indexfields = ['Submitter name', 'Submit data', 'Report state']

class DbaseComment(Dbase):
    dirname     = 'DbaseFilesIndexed/commentDB/'
    filename    = dirname + 'indexes'
    indexfields = ['Part number', 'Report state'] 

if __name__ == '__main__':
    # self-test
    dbase = DbaseComment()
    dbase.storeItem({'Description':'spam',       'Submit mode':'',
                     'Submitter name':'Bob',     'Submitter email':''}
    dbase.storeItem({'Description':'ni',         'Submit mode':'email',
                     'Submitter name':'Brian',   'Submitter email':''}
    dbase.storeItem({'Description':'more spam',  'Submit mode':'',
                     'Submitter name':'Bob',     'Submitter email':''}
    dashes = '-'*40 + '\n'

    # old stuff
    print dashes, dbase.loadSortedTable('Submitter name')
    print dashes, dbase.loadIndexedTable('Submitter name')
    print dashes, dbase.loadIndexedTable('Submitter name')['Bob']
    print dashes, dbase.loadIndexedTable('Submitter name')['Brian']

    # new stuff
    print dashes, dbase.loadKeysList('Submitter name')
    print dashes, dbase.loadKeysList('Report state')
    print dashes, dbase.loadByKey('Submitter name', 'Bob')
    print dashes, dbase.loadByKey('Submitter name', 'Brian')
    print dashes, dbase.loadByKey('Report state',   'Not yet verified'')

    # boundary cases
    print dashes, dbase.loadByKey('Submit mode',    '')
    print dashes, dbase.loadByKey('Report state',   'Nonesuch'')
    print dashes, dbase.loadByKey('Nonesuch',       'Nonesuch'')


